@@ -28,15 +28,16 @@ fitting of a transformer is costly.
 # Illustration of ``Pipeline`` and ``GridSearchCV``
 ###############################################################################
 
-import numpy as np
 import matplotlib.pyplot as plt
+import numpy as np
+
 from sklearn.datasets import load_digits
+from sklearn.decomposition import NMF, PCA
+from sklearn.feature_selection import SelectKBest, mutual_info_classif
 from sklearn.model_selection import GridSearchCV
 from sklearn.pipeline import Pipeline
-from sklearn.svm import LinearSVC
-from sklearn.decomposition import PCA, NMF
-from sklearn.feature_selection import SelectKBest, mutual_info_classif
 from sklearn.preprocessing import MinMaxScaler
+from sklearn.svm import LinearSVC
 
 X, y = load_digits(return_X_y=True)
 
@@ -103,9 +104,10 @@ plt.show()
 #     cache. Hence, use the ``memory`` constructor parameter when the fitting
 #     of a transformer is costly.
 
-from joblib import Memory
 from shutil import rmtree
 
+from joblib import Memory
+
 # Create a temporary folder to store the transformers of the pipeline
 location = "cachedir"
 memory = Memory(location=location, verbose=10)
