@@ -103,6 +103,7 @@ X[-n_outliers:] = np.dot(np.random.randn(n_outliers, n_features), outliers_cov)
 # designed to have a much larger variance in feature 2.
 
 import matplotlib.pyplot as plt
+
 from sklearn.covariance import EmpiricalCovariance, MinCovDet
 
 # fit a MCD robust estimator to data
@@ -121,6 +122,7 @@ print(
 # MCD based Mahalanobis distances fit the inlier black points much better,
 # whereas the MLE based distances are more influenced by the outlier
 # red points.
+import matplotlib.lines as mlines
 
 fig, ax = plt.subplots(figsize=(10, 5))
 # Plot data set
@@ -153,8 +155,8 @@ robust_contour = ax.contour(
 # Add legend
 ax.legend(
     [
-        emp_cov_contour.collections[1],
-        robust_contour.collections[1],
+        mlines.Line2D([], [], color="tab:blue", linestyle="dashed"),
+        mlines.Line2D([], [], color="tab:orange", linestyle="dotted"),
         inlier_plot,
         outlier_plot,
     ],
