@@ -1,7 +1,7 @@
 """
-=======================================================
-Gaussian process regression (GPR) on Mauna Loa CO2 data
-=======================================================
+====================================================================================
+Forecasting of CO2 level on Mona Loa dataset using Gaussian process regression (GPR)
+====================================================================================
 
 This example is based on Section 5.4.3 of "Gaussian Processes for Machine
 Learning" [RW2006]_. It illustrates an example of complex kernel engineering
@@ -36,7 +36,7 @@ print(__doc__)
 # in OpenML.
 from sklearn.datasets import fetch_openml
 
-co2 = fetch_openml(data_id=41187, as_frame=True, parser="pandas")
+co2 = fetch_openml(data_id=41187, as_frame=True)
 co2.frame.head()
 
 # %%
@@ -66,7 +66,15 @@ _ = plt.title("Raw air samples measurements from the Mauna Loa Observatory")
 # We will preprocess the dataset by taking a monthly average and drop month
 # for which no measurements were collected. Such a processing will have an
 # smoothing effect on the data.
-co2_data = co2_data.resample("M").mean().dropna(axis="index", how="any")
+
+try:
+    co2_data_resampled_monthly = co2_data.resample("ME")
+except ValueError:
+    # pandas < 2.2 uses M instead of ME
+    co2_data_resampled_monthly = co2_data.resample("M")
+
+
+co2_data = co2_data_resampled_monthly.mean().dropna(axis="index", how="any")
 co2_data.plot()
 plt.ylabel("Monthly average of CO$_2$ concentration (ppm)")
 _ = plt.title(
@@ -172,6 +180,7 @@ gaussian_process.fit(X, y - y_mean)
 # Thus, we create synthetic data from 1958 to the current month. In addition,
 # we need to add the subtracted mean computed during training.
 import datetime
+
 import numpy as np
 
 today = datetime.datetime.now()
