@@ -32,7 +32,7 @@ policyholders.
 
 .. [1]  A. Noll, R. Salzmann and M.V. Wuthrich, Case Study: French Motor
     Third-Party Liability Claims (November 8, 2018). `doi:10.2139/ssrn.3164764
-    <http://dx.doi.org/10.2139/ssrn.3164764>`_
+    <https://doi.org/10.2139/ssrn.3164764>`_
 
 """
 
@@ -41,22 +41,19 @@ policyholders.
 #          Olivier Grisel <olivier.grisel@ensta.org>
 # License: BSD 3 clause
 
-import numpy as np
 import matplotlib.pyplot as plt
+import numpy as np
 import pandas as pd
 
-
 ##############################################################################
 # The French Motor Third-Party Liability Claims dataset
 # -----------------------------------------------------
 #
 # Let's load the motor claim dataset from OpenML:
 # https://www.openml.org/d/41214
-
 from sklearn.datasets import fetch_openml
 
-
-df = fetch_openml(data_id=41214, as_frame=True, parser="pandas").frame
+df = fetch_openml(data_id=41214, as_frame=True).frame
 df
 
 # %%
@@ -97,11 +94,14 @@ _ = df["Frequency"].hist(bins=30, log=True, ax=ax2)
 # In order to fit linear models with those predictors it is therefore
 # necessary to perform standard feature transformations as follows:
 
-from sklearn.pipeline import make_pipeline
-from sklearn.preprocessing import FunctionTransformer, OneHotEncoder
-from sklearn.preprocessing import StandardScaler, KBinsDiscretizer
 from sklearn.compose import ColumnTransformer
-
+from sklearn.pipeline import make_pipeline
+from sklearn.preprocessing import (
+    FunctionTransformer,
+    KBinsDiscretizer,
+    OneHotEncoder,
+    StandardScaler,
+)
 
 log_scale_transformer = make_pipeline(
     FunctionTransformer(np.log, validate=False), StandardScaler()
@@ -139,8 +139,8 @@ linear_model_preprocessor = ColumnTransformer(
 # the training sample.
 
 from sklearn.dummy import DummyRegressor
-from sklearn.pipeline import Pipeline
 from sklearn.model_selection import train_test_split
+from sklearn.pipeline import Pipeline
 
 df_train, df_test = train_test_split(df, test_size=0.33, random_state=0)
 
@@ -156,9 +156,11 @@ dummy = Pipeline(
 # Let's compute the performance of this constant prediction baseline with 3
 # different regression metrics:
 
-from sklearn.metrics import mean_squared_error
-from sklearn.metrics import mean_absolute_error
-from sklearn.metrics import mean_poisson_deviance
+from sklearn.metrics import (
+    mean_absolute_error,
+    mean_poisson_deviance,
+    mean_squared_error,
+)
 
 
 def score_estimator(estimator, df_test):
@@ -207,13 +209,12 @@ score_estimator(dummy, df_test)
 # ---------------------------
 #
 # We start by modeling the target variable with the (l2 penalized) least
-# squares linear regression model, more comonly known as Ridge regression. We
+# squares linear regression model, more commonly known as Ridge regression. We
 # use a low penalization `alpha`, as we expect such a linear model to under-fit
 # on such a large dataset.
 
 from sklearn.linear_model import Ridge
 
-
 ridge_glm = Pipeline(
     [
         ("preprocessor", linear_model_preprocessor),
@@ -285,7 +286,6 @@ score_estimator(poisson_glm, df_test)
 from sklearn.ensemble import HistGradientBoostingRegressor
 from sklearn.preprocessing import OrdinalEncoder
 
-
 tree_preprocessor = ColumnTransformer(
     [
         (
