@@ -23,9 +23,8 @@ import numpy as np
 from matplotlib import pyplot as plt
 
 from sklearn.datasets import make_hastie_10_2
+from sklearn.metrics import accuracy_score, make_scorer
 from sklearn.model_selection import GridSearchCV
-from sklearn.metrics import make_scorer
-from sklearn.metrics import accuracy_score
 from sklearn.tree import DecisionTreeClassifier
 
 # %%
