@@ -37,10 +37,11 @@ over the number of true positives plus the number of false negatives
 
 :math:`R = \\frac{T_p}{T_p + F_n}`
 
-These quantities are also related to the (:math:`F_1`) score, which is defined
-as the harmonic mean of precision and recall.
+These quantities are also related to the :math:`F_1` score, which is the
+harmonic mean of precision and recall. Thus, we can compute the :math:`F_1`
+using the following formula:
 
-:math:`F1 = 2\\frac{P \\times R}{P+R}`
+:math:`F_1 = \\frac{2T_p}{2T_p + F_p + F_n}`
 
 Note that the precision may not decrease with recall. The
 definition of precision (:math:`\\frac{T_p}{T_p + F_p}`) shows that lowering
@@ -100,6 +101,7 @@ matrix as a binary prediction (micro-averaging).
 #
 # We will use a Linear SVC classifier to differentiate two types of irises.
 import numpy as np
+
 from sklearn.datasets import load_iris
 from sklearn.model_selection import train_test_split
 
@@ -123,7 +125,9 @@ from sklearn.pipeline import make_pipeline
 from sklearn.preprocessing import StandardScaler
 from sklearn.svm import LinearSVC
 
-classifier = make_pipeline(StandardScaler(), LinearSVC(random_state=random_state))
+classifier = make_pipeline(
+    StandardScaler(), LinearSVC(random_state=random_state, dual="auto")
+)
 classifier.fit(X_train, y_train)
 
 # %%
@@ -142,7 +146,7 @@ classifier.fit(X_train, y_train)
 from sklearn.metrics import PrecisionRecallDisplay
 
 display = PrecisionRecallDisplay.from_estimator(
-    classifier, X_test, y_test, name="LinearSVC"
+    classifier, X_test, y_test, name="LinearSVC", plot_chance_level=True
 )
 _ = display.ax_.set_title("2-class Precision-Recall curve")
 
@@ -152,7 +156,9 @@ _ = display.ax_.set_title("2-class Precision-Recall curve")
 # :func:`~sklearn.metrics.PrecisionRecallDisplay.from_predictions`.
 y_score = classifier.decision_function(X_test)
 
-display = PrecisionRecallDisplay.from_predictions(y_test, y_score, name="LinearSVC")
+display = PrecisionRecallDisplay.from_predictions(
+    y_test, y_score, name="LinearSVC", plot_chance_level=True
+)
 _ = display.ax_.set_title("2-class Precision-Recall curve")
 
 # %%
@@ -185,7 +191,7 @@ X_train, X_test, Y_train, Y_test = train_test_split(
 from sklearn.multiclass import OneVsRestClassifier
 
 classifier = OneVsRestClassifier(
-    make_pipeline(StandardScaler(), LinearSVC(random_state=random_state))
+    make_pipeline(StandardScaler(), LinearSVC(random_state=random_state, dual="auto"))
 )
 classifier.fit(X_train, Y_train)
 y_score = classifier.decision_function(X_test)
@@ -194,8 +200,7 @@ y_score = classifier.decision_function(X_test)
 # %%
 # The average precision score in multi-label settings
 # ...................................................
-from sklearn.metrics import precision_recall_curve
-from sklearn.metrics import average_precision_score
+from sklearn.metrics import average_precision_score, precision_recall_curve
 
 # For each class
 precision = dict()
@@ -214,20 +219,24 @@ average_precision["micro"] = average_precision_score(Y_test, y_score, average="m
 # %%
 # Plot the micro-averaged Precision-Recall curve
 # ..............................................
+from collections import Counter
+
 display = PrecisionRecallDisplay(
     recall=recall["micro"],
     precision=precision["micro"],
     average_precision=average_precision["micro"],
+    prevalence_pos_label=Counter(Y_test.ravel())[1] / Y_test.size,
 )
-display.plot()
+display.plot(plot_chance_level=True)
 _ = display.ax_.set_title("Micro-averaged over all classes")
 
 # %%
 # Plot Precision-Recall curve for each class and iso-f1 curves
 # ............................................................
-import matplotlib.pyplot as plt
 from itertools import cycle
 
+import matplotlib.pyplot as plt
+
 # setup plot details
 colors = cycle(["navy", "turquoise", "darkorange", "cornflowerblue", "teal"])
 
@@ -261,8 +270,6 @@ handles, labels = display.ax_.get_legend_handles_labels()
 handles.extend([l])
 labels.extend(["iso-f1 curves"])
 # set the legend and the axes
-ax.set_xlim([0.0, 1.0])
-ax.set_ylim([0.0, 1.05])
 ax.legend(handles=handles, labels=labels, loc="best")
 ax.set_title("Extension of Precision-Recall curve to multi-class")
 
