@@ -5,27 +5,28 @@
 
 import math
 import re
-import pytest
+
 import numpy as np
-import scipy.sparse as sp
+import pytest
 from scipy.special import logsumexp
 
 from sklearn._loss.loss import HalfMultinomialLoss
+from sklearn.base import clone
+from sklearn.datasets import load_iris, make_blobs, make_classification
+from sklearn.linear_model import LogisticRegression, Ridge
+from sklearn.linear_model._base import make_dataset
 from sklearn.linear_model._linear_loss import LinearModelLoss
 from sklearn.linear_model._sag import get_auto_step_size
 from sklearn.linear_model._sag_fast import _multinomial_grad_loss_all_samples
-from sklearn.linear_model import LogisticRegression, Ridge
-from sklearn.linear_model._base import make_dataset
-
+from sklearn.preprocessing import LabelBinarizer, LabelEncoder
+from sklearn.utils import check_random_state, compute_class_weight
+from sklearn.utils._testing import (
+    assert_allclose,
+    assert_almost_equal,
+    assert_array_almost_equal,
+)
 from sklearn.utils.extmath import row_norms
-from sklearn.utils._testing import assert_almost_equal
-from sklearn.utils._testing import assert_array_almost_equal
-from sklearn.utils._testing import assert_allclose
-from sklearn.utils import compute_class_weight
-from sklearn.utils import check_random_state
-from sklearn.preprocessing import LabelEncoder, LabelBinarizer
-from sklearn.datasets import make_blobs, load_iris, make_classification
-from sklearn.base import clone
+from sklearn.utils.fixes import CSR_CONTAINERS
 
 iris = load_iris()
 
@@ -95,7 +96,7 @@ def sag(
 
     for epoch in range(n_iter):
         for k in range(n_samples):
-            idx = int(rng.rand(1) * n_samples)
+            idx = int(rng.rand() * n_samples)
             # idx = k
             entry = X[idx]
             seen.add(idx)
@@ -167,7 +168,7 @@ def sag_sparse(
     for epoch in range(n_iter):
         for k in range(n_samples):
             # idx = k
-            idx = int(rng.rand(1) * n_samples)
+            idx = int(rng.rand() * n_samples)
             entry = X[idx]
             seen.add(idx)
 
@@ -355,7 +356,8 @@ def test_regressor_matching():
 
 
 @pytest.mark.filterwarnings("ignore:The max_iter was reached")
-def test_sag_pobj_matches_logistic_regression():
+@pytest.mark.parametrize("csr_container", CSR_CONTAINERS)
+def test_sag_pobj_matches_logistic_regression(csr_container):
     """tests if the sag pobj matches log reg"""
     n_samples = 100
     alpha = 1.0
@@ -382,7 +384,7 @@ def test_sag_pobj_matches_logistic_regression():
     )
 
     clf1.fit(X, y)
-    clf2.fit(sp.csr_matrix(X), y)
+    clf2.fit(csr_container(X), y)
     clf3.fit(X, y)
 
     pobj1 = get_pobj(clf1.coef_, alpha, X, y, log_loss)
@@ -395,7 +397,8 @@ def test_sag_pobj_matches_logistic_regression():
 
 
 @pytest.mark.filterwarnings("ignore:The max_iter was reached")
-def test_sag_pobj_matches_ridge_regression():
+@pytest.mark.parametrize("csr_container", CSR_CONTAINERS)
+def test_sag_pobj_matches_ridge_regression(csr_container):
     """tests if the sag pobj matches ridge reg"""
     n_samples = 100
     n_features = 10
@@ -426,7 +429,7 @@ def test_sag_pobj_matches_ridge_regression():
     )
 
     clf1.fit(X, y)
-    clf2.fit(sp.csr_matrix(X), y)
+    clf2.fit(csr_container(X), y)
     clf3.fit(X, y)
 
     pobj1 = get_pobj(clf1.coef_, alpha, X, y, squared_loss)
@@ -439,7 +442,8 @@ def test_sag_pobj_matches_ridge_regression():
 
 
 @pytest.mark.filterwarnings("ignore:The max_iter was reached")
-def test_sag_regressor_computed_correctly():
+@pytest.mark.parametrize("csr_container", CSR_CONTAINERS)
+def test_sag_regressor_computed_correctly(csr_container):
     """tests if the sag regressor is computed correctly"""
     alpha = 0.1
     n_features = 10
@@ -464,7 +468,7 @@ def test_sag_regressor_computed_correctly():
     clf2 = clone(clf1)
 
     clf1.fit(X, y)
-    clf2.fit(sp.csr_matrix(X), y)
+    clf2.fit(csr_container(X), y)
 
     spweights1, spintercept1 = sag_sparse(
         X,
@@ -550,7 +554,8 @@ def test_get_auto_step_size():
 
 
 @pytest.mark.parametrize("seed", range(3))  # locally tested with 1000 seeds
-def test_sag_regressor(seed):
+@pytest.mark.parametrize("csr_container", CSR_CONTAINERS)
+def test_sag_regressor(seed, csr_container):
     """tests if the sag regressor performs well"""
     xmin, xmax = -5, 5
     n_samples = 300
@@ -572,7 +577,7 @@ def test_sag_regressor(seed):
     )
     clf2 = clone(clf1)
     clf1.fit(X, y)
-    clf2.fit(sp.csr_matrix(X), y)
+    clf2.fit(csr_container(X), y)
     score1 = clf1.score(X, y)
     score2 = clf2.score(X, y)
     assert score1 > 0.98
@@ -584,7 +589,7 @@ def test_sag_regressor(seed):
     clf1 = Ridge(tol=tol, solver="sag", max_iter=max_iter, alpha=alpha * n_samples)
     clf2 = clone(clf1)
     clf1.fit(X, y)
-    clf2.fit(sp.csr_matrix(X), y)
+    clf2.fit(csr_container(X), y)
     score1 = clf1.score(X, y)
     score2 = clf2.score(X, y)
     assert score1 > 0.45
@@ -592,7 +597,8 @@ def test_sag_regressor(seed):
 
 
 @pytest.mark.filterwarnings("ignore:The max_iter was reached")
-def test_sag_classifier_computed_correctly():
+@pytest.mark.parametrize("csr_container", CSR_CONTAINERS)
+def test_sag_classifier_computed_correctly(csr_container):
     """tests if the binary classifier is computed correctly"""
     alpha = 0.1
     n_samples = 50
@@ -618,7 +624,7 @@ def test_sag_classifier_computed_correctly():
     clf2 = clone(clf1)
 
     clf1.fit(X, y)
-    clf2.fit(sp.csr_matrix(X), y)
+    clf2.fit(csr_container(X), y)
 
     spweights, spintercept = sag_sparse(
         X,
@@ -648,7 +654,8 @@ def test_sag_classifier_computed_correctly():
 
 
 @pytest.mark.filterwarnings("ignore:The max_iter was reached")
-def test_sag_multiclass_computed_correctly():
+@pytest.mark.parametrize("csr_container", CSR_CONTAINERS)
+def test_sag_multiclass_computed_correctly(csr_container):
     """tests if the multiclass classifier is computed correctly"""
     alpha = 0.1
     n_samples = 20
@@ -671,7 +678,7 @@ def test_sag_multiclass_computed_correctly():
     clf2 = clone(clf1)
 
     clf1.fit(X, y)
-    clf2.fit(sp.csr_matrix(X), y)
+    clf2.fit(csr_container(X), y)
 
     coef1 = []
     intercept1 = []
@@ -719,7 +726,8 @@ def test_sag_multiclass_computed_correctly():
         assert_almost_equal(clf2.intercept_[i], intercept2[i], decimal=1)
 
 
-def test_classifier_results():
+@pytest.mark.parametrize("csr_container", CSR_CONTAINERS)
+def test_classifier_results(csr_container):
     """tests if classifier results match target"""
     alpha = 0.1
     n_features = 20
@@ -741,7 +749,7 @@ def test_classifier_results():
     clf2 = clone(clf1)
 
     clf1.fit(X, y)
-    clf2.fit(sp.csr_matrix(X), y)
+    clf2.fit(csr_container(X), y)
     pred1 = clf1.predict(X)
     pred2 = clf2.predict(X)
     assert_almost_equal(pred1, y, decimal=12)
@@ -749,7 +757,8 @@ def test_classifier_results():
 
 
 @pytest.mark.filterwarnings("ignore:The max_iter was reached")
-def test_binary_classifier_class_weight():
+@pytest.mark.parametrize("csr_container", CSR_CONTAINERS)
+def test_binary_classifier_class_weight(csr_container):
     """tests binary classifier with classweights for each class"""
     alpha = 0.1
     n_samples = 50
@@ -777,7 +786,7 @@ def test_binary_classifier_class_weight():
     clf2 = clone(clf1)
 
     clf1.fit(X, y)
-    clf2.fit(sp.csr_matrix(X), y)
+    clf2.fit(csr_container(X), y)
 
     le = LabelEncoder()
     class_weight_ = compute_class_weight(class_weight, classes=np.unique(y), y=y)
@@ -812,7 +821,8 @@ def test_binary_classifier_class_weight():
 
 
 @pytest.mark.filterwarnings("ignore:The max_iter was reached")
-def test_multiclass_classifier_class_weight():
+@pytest.mark.parametrize("csr_container", CSR_CONTAINERS)
+def test_multiclass_classifier_class_weight(csr_container):
     """tests multiclass with classweights for each class"""
     alpha = 0.1
     n_samples = 20
@@ -836,7 +846,7 @@ def test_multiclass_classifier_class_weight():
     )
     clf2 = clone(clf1)
     clf1.fit(X, y)
-    clf2.fit(sp.csr_matrix(X), y)
+    clf2.fit(csr_container(X), y)
 
     le = LabelEncoder()
     class_weight_ = compute_class_weight(class_weight, classes=np.unique(y), y=y)
@@ -925,8 +935,7 @@ def test_multinomial_loss():
     rng = check_random_state(42)
     weights = rng.randn(n_features, n_classes)
     intercept = rng.randn(n_classes)
-    sample_weights = rng.randn(n_samples)
-    np.abs(sample_weights, sample_weights)
+    sample_weights = np.abs(rng.randn(n_samples))
 
     # compute loss and gradient like in multinomial SAG
     dataset, _ = make_dataset(X, y, sample_weights, random_state=42)
@@ -943,6 +952,9 @@ def test_multinomial_loss():
         weights_intercept, X, y, l2_reg_strength=0.0, sample_weight=sample_weights
     )
     grad_2 = grad_2[:, :-1].T
+    # convert to same convention, i.e. LinearModelLoss uses average(loss, weight=sw)
+    loss_2 *= np.sum(sample_weights)
+    grad_2 *= np.sum(sample_weights)
 
     # comparison
     assert_array_almost_equal(grad_1, grad_2)
@@ -977,6 +989,9 @@ def test_multinomial_loss_ground_truth():
         weights_intercept, X, y, l2_reg_strength=0.0, sample_weight=sample_weights
     )
     grad_2 = grad_2[:, :-1].T
+    # convert to same convention, i.e. LinearModelLoss uses average(loss, weight=sw)
+    loss_2 *= np.sum(sample_weights)
+    grad_2 *= np.sum(sample_weights)
 
     assert_almost_equal(loss_1, loss_2)
     assert_array_almost_equal(grad_1, grad_2)
