@@ -1,20 +1,18 @@
-
-import pytest
-
 import numpy as np
+import pytest
 import scipy.sparse as sp
 
 from sklearn.base import clone
-from sklearn.utils._testing import assert_array_equal
-from sklearn.utils._testing import assert_array_almost_equal
-from sklearn.utils._testing import assert_almost_equal
-from sklearn.utils._testing import assert_raises
-from sklearn.utils._testing import assert_warns_message
-from sklearn.utils._testing import ignore_warnings
-from sklearn.utils.stats import _weighted_percentile
-
 from sklearn.dummy import DummyClassifier, DummyRegressor
 from sklearn.exceptions import NotFittedError
+from sklearn.utils._testing import (
+    assert_almost_equal,
+    assert_array_almost_equal,
+    assert_array_equal,
+    ignore_warnings,
+)
+from sklearn.utils.fixes import CSC_CONTAINERS
+from sklearn.utils.stats import _weighted_percentile
 
 
 @ignore_warnings
@@ -52,10 +50,7 @@ def _check_behavior_2d(clf):
     assert y.shape == y_pred.shape
 
     # 2d case
-    y = np.array([[1, 0],
-                  [2, 0],
-                  [1, 0],
-                  [1, 3]])
+    y = np.array([[1, 0], [2, 0], [1, 0], [1, 3]])
     est = clone(clf)
     est.fit(X, y)
     y_pred = est.predict(X)
@@ -65,22 +60,16 @@ def _check_behavior_2d(clf):
 def _check_behavior_2d_for_constant(clf):
     # 2d case only
     X = np.array([[0], [0], [0], [0]])  # ignored
-    y = np.array([[1, 0, 5, 4, 3],
-                  [2, 0, 1, 2, 5],
-                  [1, 0, 4, 5, 2],
-                  [1, 3, 3, 2, 0]])
+    y = np.array([[1, 0, 5, 4, 3], [2, 0, 1, 2, 5], [1, 0, 4, 5, 2], [1, 3, 3, 2, 0]])
     est = clone(clf)
     est.fit(X, y)
     y_pred = est.predict(X)
     assert y.shape == y_pred.shape
 
 
-def _check_equality_regressor(statistic, y_learn, y_pred_learn,
-                              y_test, y_pred_test):
-    assert_array_almost_equal(np.tile(statistic, (y_learn.shape[0], 1)),
-                              y_pred_learn)
-    assert_array_almost_equal(np.tile(statistic, (y_test.shape[0], 1)),
-                              y_pred_test)
+def _check_equality_regressor(statistic, y_learn, y_pred_learn, y_test, y_pred_test):
+    assert_array_almost_equal(np.tile(statistic, (y_learn.shape[0], 1)), y_pred_learn)
+    assert_array_almost_equal(np.tile(statistic, (y_test.shape[0], 1)), y_pred_test)
 
 
 def test_most_frequent_and_prior_strategy():
@@ -94,11 +83,13 @@ def test_most_frequent_and_prior_strategy():
         _check_predict_proba(clf, X, y)
 
         if strategy == "prior":
-            assert_array_almost_equal(clf.predict_proba([X[0]]),
-                                      clf.class_prior_.reshape((1, -1)))
+            assert_array_almost_equal(
+                clf.predict_proba([X[0]]), clf.class_prior_.reshape((1, -1))
+            )
         else:
-            assert_array_almost_equal(clf.predict_proba([X[0]]),
-                                      clf.class_prior_.reshape((1, -1)) > 0.5)
+            assert_array_almost_equal(
+                clf.predict_proba([X[0]]), clf.class_prior_.reshape((1, -1)) > 0.5
+            )
 
 
 def test_most_frequent_and_prior_strategy_with_2d_column_y():
@@ -119,46 +110,40 @@ def test_most_frequent_and_prior_strategy_with_2d_column_y():
 
 def test_most_frequent_and_prior_strategy_multioutput():
     X = [[0], [0], [0], [0]]  # ignored
-    y = np.array([[1, 0],
-                  [2, 0],
-                  [1, 0],
-                  [1, 3]])
+    y = np.array([[1, 0], [2, 0], [1, 0], [1, 3]])
 
     n_samples = len(X)
 
     for strategy in ("prior", "most_frequent"):
         clf = DummyClassifier(strategy=strategy, random_state=0)
         clf.fit(X, y)
-        assert_array_equal(clf.predict(X),
-                           np.hstack([np.ones((n_samples, 1)),
-                                      np.zeros((n_samples, 1))]))
+        assert_array_equal(
+            clf.predict(X),
+            np.hstack([np.ones((n_samples, 1)), np.zeros((n_samples, 1))]),
+        )
         _check_predict_proba(clf, X, y)
         _check_behavior_2d(clf)
 
 
-def test_stratified_strategy():
+def test_stratified_strategy(global_random_seed):
     X = [[0]] * 5  # ignored
     y = [1, 2, 1, 1, 2]
-    clf = DummyClassifier(strategy="stratified", random_state=0)
+    clf = DummyClassifier(strategy="stratified", random_state=global_random_seed)
     clf.fit(X, y)
 
     X = [[0]] * 500
     y_pred = clf.predict(X)
     p = np.bincount(y_pred) / float(len(X))
-    assert_almost_equal(p[1], 3. / 5, decimal=1)
-    assert_almost_equal(p[2], 2. / 5, decimal=1)
+    assert_almost_equal(p[1], 3.0 / 5, decimal=1)
+    assert_almost_equal(p[2], 2.0 / 5, decimal=1)
     _check_predict_proba(clf, X, y)
 
 
-def test_stratified_strategy_multioutput():
+def test_stratified_strategy_multioutput(global_random_seed):
     X = [[0]] * 5  # ignored
-    y = np.array([[2, 1],
-                  [2, 2],
-                  [1, 1],
-                  [1, 2],
-                  [1, 1]])
+    y = np.array([[2, 1], [2, 2], [1, 1], [1, 2], [1, 1]])
 
-    clf = DummyClassifier(strategy="stratified", random_state=0)
+    clf = DummyClassifier(strategy="stratified", random_state=global_random_seed)
     clf.fit(X, y)
 
     X = [[0]] * 500
@@ -166,17 +151,17 @@ def test_stratified_strategy_multioutput():
 
     for k in range(y.shape[1]):
         p = np.bincount(y_pred[:, k]) / float(len(X))
-        assert_almost_equal(p[1], 3. / 5, decimal=1)
-        assert_almost_equal(p[2], 2. / 5, decimal=1)
+        assert_almost_equal(p[1], 3.0 / 5, decimal=1)
+        assert_almost_equal(p[2], 2.0 / 5, decimal=1)
         _check_predict_proba(clf, X, y)
 
     _check_behavior_2d(clf)
 
 
-def test_uniform_strategy():
+def test_uniform_strategy(global_random_seed):
     X = [[0]] * 4  # ignored
     y = [1, 2, 1, 1]
-    clf = DummyClassifier(strategy="uniform", random_state=0)
+    clf = DummyClassifier(strategy="uniform", random_state=global_random_seed)
     clf.fit(X, y)
 
     X = [[0]] * 500
@@ -187,13 +172,10 @@ def test_uniform_strategy():
     _check_predict_proba(clf, X, y)
 
 
-def test_uniform_strategy_multioutput():
+def test_uniform_strategy_multioutput(global_random_seed):
     X = [[0]] * 4  # ignored
-    y = np.array([[2, 1],
-                  [2, 2],
-                  [1, 2],
-                  [1, 1]])
-    clf = DummyClassifier(strategy="uniform", random_state=0)
+    y = np.array([[2, 1], [2, 2], [1, 2], [1, 1]])
+    clf = DummyClassifier(strategy="uniform", random_state=global_random_seed)
     clf.fit(X, y)
 
     X = [[0]] * 500
@@ -216,56 +198,46 @@ def test_string_labels():
     assert_array_equal(clf.predict(X), ["paris"] * 5)
 
 
-@pytest.mark.parametrize("y,y_test", [
-    ([2, 1, 1, 1], [2, 2, 1, 1]),
-    (np.array([[2, 2],
-               [1, 1],
-               [1, 1],
-               [1, 1]]),
-     np.array([[2, 2],
-               [2, 2],
-               [1, 1],
-               [1, 1]]))
-])
+@pytest.mark.parametrize(
+    "y,y_test",
+    [
+        ([2, 1, 1, 1], [2, 2, 1, 1]),
+        (
+            np.array([[2, 2], [1, 1], [1, 1], [1, 1]]),
+            np.array([[2, 2], [2, 2], [1, 1], [1, 1]]),
+        ),
+    ],
+)
 def test_classifier_score_with_None(y, y_test):
     clf = DummyClassifier(strategy="most_frequent")
     clf.fit(None, y)
     assert clf.score(None, y_test) == 0.5
 
 
-@pytest.mark.parametrize("strategy", [
-    "stratified",
-    "most_frequent",
-    "prior",
-    "uniform",
-    "constant"
-])
-def test_classifier_prediction_independent_of_X(strategy):
+@pytest.mark.parametrize(
+    "strategy", ["stratified", "most_frequent", "prior", "uniform", "constant"]
+)
+def test_classifier_prediction_independent_of_X(strategy, global_random_seed):
     y = [0, 2, 1, 1]
     X1 = [[0]] * 4
-    clf1 = DummyClassifier(strategy=strategy, random_state=0, constant=0)
+    clf1 = DummyClassifier(
+        strategy=strategy, random_state=global_random_seed, constant=0
+    )
     clf1.fit(X1, y)
     predictions1 = clf1.predict(X1)
 
     X2 = [[1]] * 4
-    clf2 = DummyClassifier(strategy=strategy, random_state=0, constant=0)
+    clf2 = DummyClassifier(
+        strategy=strategy, random_state=global_random_seed, constant=0
+    )
     clf2.fit(X2, y)
     predictions2 = clf2.predict(X2)
 
     assert_array_equal(predictions1, predictions2)
 
 
-def test_classifier_exceptions():
-    clf = DummyClassifier(strategy="unknown")
-    assert_raises(ValueError, clf.fit, [], [])
-
-    assert_raises(NotFittedError, clf.predict, [])
-    assert_raises(NotFittedError, clf.predict_proba, [])
-
-
-def test_mean_strategy_regressor():
-
-    random_state = np.random.RandomState(seed=1)
+def test_mean_strategy_regressor(global_random_seed):
+    random_state = np.random.RandomState(seed=global_random_seed)
 
     X = [[0]] * 4  # ignored
     y = random_state.randn(4)
@@ -275,9 +247,8 @@ def test_mean_strategy_regressor():
     assert_array_equal(reg.predict(X), [np.mean(y)] * len(X))
 
 
-def test_mean_strategy_multioutput_regressor():
-
-    random_state = np.random.RandomState(seed=1)
+def test_mean_strategy_multioutput_regressor(global_random_seed):
+    random_state = np.random.RandomState(seed=global_random_seed)
 
     X_learn = random_state.randn(10, 10)
     y_learn = random_state.randn(10, 5)
@@ -299,12 +270,12 @@ def test_mean_strategy_multioutput_regressor():
 
 def test_regressor_exceptions():
     reg = DummyRegressor()
-    assert_raises(NotFittedError, reg.predict, [])
+    with pytest.raises(NotFittedError):
+        reg.predict([])
 
 
-def test_median_strategy_regressor():
-
-    random_state = np.random.RandomState(seed=1)
+def test_median_strategy_regressor(global_random_seed):
+    random_state = np.random.RandomState(seed=global_random_seed)
 
     X = [[0]] * 5  # ignored
     y = random_state.randn(5)
@@ -314,9 +285,8 @@ def test_median_strategy_regressor():
     assert_array_equal(reg.predict(X), [np.median(y)] * len(X))
 
 
-def test_median_strategy_multioutput_regressor():
-
-    random_state = np.random.RandomState(seed=1)
+def test_median_strategy_multioutput_regressor(global_random_seed):
+    random_state = np.random.RandomState(seed=global_random_seed)
 
     X_learn = random_state.randn(10, 10)
     y_learn = random_state.randn(10, 5)
@@ -332,14 +302,12 @@ def test_median_strategy_multioutput_regressor():
     y_pred_learn = est.predict(X_learn)
     y_pred_test = est.predict(X_test)
 
-    _check_equality_regressor(
-        median, y_learn, y_pred_learn, y_test, y_pred_test)
+    _check_equality_regressor(median, y_learn, y_pred_learn, y_test, y_pred_test)
     _check_behavior_2d(est)
 
 
-def test_quantile_strategy_regressor():
-
-    random_state = np.random.RandomState(seed=1)
+def test_quantile_strategy_regressor(global_random_seed):
+    random_state = np.random.RandomState(seed=global_random_seed)
 
     X = [[0]] * 5  # ignored
     y = random_state.randn(5)
@@ -361,9 +329,8 @@ def test_quantile_strategy_regressor():
     assert_array_equal(reg.predict(X), [np.percentile(y, q=30)] * len(X))
 
 
-def test_quantile_strategy_multioutput_regressor():
-
-    random_state = np.random.RandomState(seed=1)
+def test_quantile_strategy_multioutput_regressor(global_random_seed):
+    random_state = np.random.RandomState(seed=global_random_seed)
 
     X_learn = random_state.randn(10, 10)
     y_learn = random_state.randn(10, 5)
@@ -380,8 +347,7 @@ def test_quantile_strategy_multioutput_regressor():
     y_pred_learn = est.predict(X_learn)
     y_pred_test = est.predict(X_test)
 
-    _check_equality_regressor(
-        median, y_learn, y_pred_learn, y_test, y_pred_test)
+    _check_equality_regressor(median, y_learn, y_pred_learn, y_test, y_pred_test)
     _check_behavior_2d(est)
 
     # Correctness oracle
@@ -391,42 +357,31 @@ def test_quantile_strategy_multioutput_regressor():
     y_pred_test = est.predict(X_test)
 
     _check_equality_regressor(
-        quantile_values, y_learn, y_pred_learn, y_test, y_pred_test)
+        quantile_values, y_learn, y_pred_learn, y_test, y_pred_test
+    )
     _check_behavior_2d(est)
 
 
 def test_quantile_invalid():
-
     X = [[0]] * 5  # ignored
     y = [0] * 5  # ignored
 
-    est = DummyRegressor(strategy="quantile")
-    assert_raises(ValueError, est.fit, X, y)
-
     est = DummyRegressor(strategy="quantile", quantile=None)
-    assert_raises(ValueError, est.fit, X, y)
-
-    est = DummyRegressor(strategy="quantile", quantile=[0])
-    assert_raises(ValueError, est.fit, X, y)
-
-    est = DummyRegressor(strategy="quantile", quantile=-0.1)
-    assert_raises(ValueError, est.fit, X, y)
-
-    est = DummyRegressor(strategy="quantile", quantile=1.1)
-    assert_raises(ValueError, est.fit, X, y)
-
-    est = DummyRegressor(strategy="quantile", quantile='abc')
-    assert_raises(TypeError, est.fit, X, y)
+    err_msg = (
+        "When using `strategy='quantile', you have to specify the desired quantile"
+    )
+    with pytest.raises(ValueError, match=err_msg):
+        est.fit(X, y)
 
 
 def test_quantile_strategy_empty_train():
     est = DummyRegressor(strategy="quantile", quantile=0.4)
-    assert_raises(ValueError, est.fit, [], [])
+    with pytest.raises(ValueError):
+        est.fit([], [])
 
 
-def test_constant_strategy_regressor():
-
-    random_state = np.random.RandomState(seed=1)
+def test_constant_strategy_regressor(global_random_seed):
+    random_state = np.random.RandomState(seed=global_random_seed)
 
     X = [[0]] * 5  # ignored
     y = random_state.randn(5)
@@ -439,10 +394,12 @@ def test_constant_strategy_regressor():
     reg.fit(X, y)
     assert_array_equal(reg.predict(X), [43] * len(X))
 
+    # non-regression test for #22478
+    assert not isinstance(reg.constant, np.ndarray)
 
-def test_constant_strategy_multioutput_regressor():
 
-    random_state = np.random.RandomState(seed=1)
+def test_constant_strategy_multioutput_regressor(global_random_seed):
+    random_state = np.random.RandomState(seed=global_random_seed)
 
     X_learn = random_state.randn(10, 10)
     y_learn = random_state.randn(10, 5)
@@ -459,8 +416,7 @@ def test_constant_strategy_multioutput_regressor():
     y_pred_learn = est.predict(X_learn)
     y_pred_test = est.predict(X_test)
 
-    _check_equality_regressor(
-        constants, y_learn, y_pred_learn, y_test, y_pred_test)
+    _check_equality_regressor(constants, y_learn, y_pred_learn, y_test, y_pred_test)
     _check_behavior_2d_for_constant(est)
 
 
@@ -468,35 +424,31 @@ def test_y_mean_attribute_regressor():
     X = [[0]] * 5
     y = [1, 2, 4, 6, 8]
     # when strategy = 'mean'
-    est = DummyRegressor(strategy='mean')
+    est = DummyRegressor(strategy="mean")
     est.fit(X, y)
 
     assert est.constant_ == np.mean(y)
 
 
-def test_unknown_strategey_regressor():
-    X = [[0]] * 5
-    y = [1, 2, 4, 6, 8]
-
-    est = DummyRegressor(strategy='gona')
-    assert_raises(ValueError, est.fit, X, y)
-
-
 def test_constants_not_specified_regressor():
     X = [[0]] * 5
     y = [1, 2, 4, 6, 8]
 
-    est = DummyRegressor(strategy='constant')
-    assert_raises(TypeError, est.fit, X, y)
+    est = DummyRegressor(strategy="constant")
+    err_msg = "Constant target value has to be specified"
+    with pytest.raises(TypeError, match=err_msg):
+        est.fit(X, y)
 
 
-def test_constant_size_multioutput_regressor():
-    random_state = np.random.RandomState(seed=1)
+def test_constant_size_multioutput_regressor(global_random_seed):
+    random_state = np.random.RandomState(seed=global_random_seed)
     X = random_state.randn(10, 10)
     y = random_state.randn(10, 5)
 
-    est = DummyRegressor(strategy='constant', constant=[1, 2, 3, 4])
-    assert_raises(ValueError, est.fit, X, y)
+    est = DummyRegressor(strategy="constant", constant=[1, 2, 3, 4])
+    err_msg = r"Constant target value should have shape \(5, 1\)."
+    with pytest.raises(ValueError, match=err_msg):
+        est.fit(X, y)
 
 
 def test_constant_strategy():
@@ -509,55 +461,60 @@ def test_constant_strategy():
     _check_predict_proba(clf, X, y)
 
     X = [[0], [0], [0], [0]]  # ignored
-    y = ['two', 'one', 'two', 'two']
-    clf = DummyClassifier(strategy="constant", random_state=0, constant='one')
+    y = ["two", "one", "two", "two"]
+    clf = DummyClassifier(strategy="constant", random_state=0, constant="one")
     clf.fit(X, y)
-    assert_array_equal(clf.predict(X), np.array(['one'] * 4))
+    assert_array_equal(clf.predict(X), np.array(["one"] * 4))
     _check_predict_proba(clf, X, y)
 
 
 def test_constant_strategy_multioutput():
     X = [[0], [0], [0], [0]]  # ignored
-    y = np.array([[2, 3],
-                  [1, 3],
-                  [2, 3],
-                  [2, 0]])
+    y = np.array([[2, 3], [1, 3], [2, 3], [2, 0]])
 
     n_samples = len(X)
 
-    clf = DummyClassifier(strategy="constant", random_state=0,
-                          constant=[1, 0])
+    clf = DummyClassifier(strategy="constant", random_state=0, constant=[1, 0])
     clf.fit(X, y)
-    assert_array_equal(clf.predict(X),
-                       np.hstack([np.ones((n_samples, 1)),
-                                  np.zeros((n_samples, 1))]))
+    assert_array_equal(
+        clf.predict(X), np.hstack([np.ones((n_samples, 1)), np.zeros((n_samples, 1))])
+    )
     _check_predict_proba(clf, X, y)
 
 
-@pytest.mark.parametrize('y, params, err_msg', [
-    ([2, 1, 2, 2],
-     {'random_state': 0},
-     "Constant.*has to be specified"),
-    ([2, 1, 2, 2],
-     {'constant': [2, 0]},
-     "Constant.*should have shape"),
-    (np.transpose([[2, 1, 2, 2], [2, 1, 2, 2]]),
-     {'constant': 2},
-     "Constant.*should have shape"),
-    ([2, 1, 2, 2],
-     {'constant': 'my-constant'},
-     "constant=my-constant.*Possible values.*\\[1, 2]"),
-    (np.transpose([[2, 1, 2, 2], [2, 1, 2, 2]]),
-     {'constant': [2, 'unknown']},
-     "constant=\\[2, 'unknown'].*Possible values.*\\[1, 2]")],
-    ids=["no-constant", "too-many-constant", "not-enough-output",
-         "single-output", "multi-output"]
+@pytest.mark.parametrize(
+    "y, params, err_msg",
+    [
+        ([2, 1, 2, 2], {"random_state": 0}, "Constant.*has to be specified"),
+        ([2, 1, 2, 2], {"constant": [2, 0]}, "Constant.*should have shape"),
+        (
+            np.transpose([[2, 1, 2, 2], [2, 1, 2, 2]]),
+            {"constant": 2},
+            "Constant.*should have shape",
+        ),
+        (
+            [2, 1, 2, 2],
+            {"constant": "my-constant"},
+            "constant=my-constant.*Possible values.*\\[1, 2]",
+        ),
+        (
+            np.transpose([[2, 1, 2, 2], [2, 1, 2, 2]]),
+            {"constant": [2, "unknown"]},
+            "constant=\\[2, 'unknown'].*Possible values.*\\[1, 2]",
+        ),
+    ],
+    ids=[
+        "no-constant",
+        "too-many-constant",
+        "not-enough-output",
+        "single-output",
+        "multi-output",
+    ],
 )
 def test_constant_strategy_exceptions(y, params, err_msg):
     X = [[0], [0], [0], [0]]
 
     clf = DummyClassifier(strategy="constant", **params)
-
     with pytest.raises(ValueError, match=err_msg):
         clf.fit(X, y)
 
@@ -565,19 +522,16 @@ def test_constant_strategy_exceptions(y, params, err_msg):
 def test_classification_sample_weight():
     X = [[0], [0], [1]]
     y = [0, 1, 0]
-    sample_weight = [0.1, 1., 0.1]
+    sample_weight = [0.1, 1.0, 0.1]
 
     clf = DummyClassifier(strategy="stratified").fit(X, y, sample_weight)
-    assert_array_almost_equal(clf.class_prior_, [0.2 / 1.2, 1. / 1.2])
+    assert_array_almost_equal(clf.class_prior_, [0.2 / 1.2, 1.0 / 1.2])
 
 
-def test_constant_strategy_sparse_target():
+@pytest.mark.parametrize("csc_container", CSC_CONTAINERS)
+def test_constant_strategy_sparse_target(csc_container):
     X = [[0]] * 5  # ignored
-    y = sp.csc_matrix(np.array([[0, 1],
-                                [4, 0],
-                                [1, 1],
-                                [1, 4],
-                                [1, 1]]))
+    y = csc_container(np.array([[0, 1], [4, 0], [1, 1], [1, 4], [1, 1]]))
 
     n_samples = len(X)
 
@@ -585,22 +539,19 @@ def test_constant_strategy_sparse_target():
     clf.fit(X, y)
     y_pred = clf.predict(X)
     assert sp.issparse(y_pred)
-    assert_array_equal(y_pred.toarray(), np.hstack([np.ones((n_samples, 1)),
-                                                    np.zeros((n_samples, 1))]))
+    assert_array_equal(
+        y_pred.toarray(), np.hstack([np.ones((n_samples, 1)), np.zeros((n_samples, 1))])
+    )
 
 
-def test_uniform_strategy_sparse_target_warning():
+@pytest.mark.parametrize("csc_container", CSC_CONTAINERS)
+def test_uniform_strategy_sparse_target_warning(global_random_seed, csc_container):
     X = [[0]] * 5  # ignored
-    y = sp.csc_matrix(np.array([[2, 1],
-                                [2, 2],
-                                [1, 4],
-                                [4, 2],
-                                [1, 1]]))
+    y = csc_container(np.array([[2, 1], [2, 2], [1, 4], [4, 2], [1, 1]]))
 
-    clf = DummyClassifier(strategy="uniform", random_state=0)
-    assert_warns_message(UserWarning,
-                         "the uniform strategy would not save memory",
-                         clf.fit, X, y)
+    clf = DummyClassifier(strategy="uniform", random_state=global_random_seed)
+    with pytest.warns(UserWarning, match="the uniform strategy would not save memory"):
+        clf.fit(X, y)
 
     X = [[0]] * 500
     y_pred = clf.predict(X)
@@ -612,15 +563,12 @@ def test_uniform_strategy_sparse_target_warning():
         assert_almost_equal(p[4], 1 / 3, decimal=1)
 
 
-def test_stratified_strategy_sparse_target():
+@pytest.mark.parametrize("csc_container", CSC_CONTAINERS)
+def test_stratified_strategy_sparse_target(global_random_seed, csc_container):
     X = [[0]] * 5  # ignored
-    y = sp.csc_matrix(np.array([[4, 1],
-                                [0, 0],
-                                [1, 1],
-                                [1, 4],
-                                [1, 1]]))
+    y = csc_container(np.array([[4, 1], [0, 0], [1, 1], [1, 4], [1, 1]]))
 
-    clf = DummyClassifier(strategy="stratified", random_state=0)
+    clf = DummyClassifier(strategy="stratified", random_state=global_random_seed)
     clf.fit(X, y)
 
     X = [[0]] * 500
@@ -630,18 +578,15 @@ def test_stratified_strategy_sparse_target():
 
     for k in range(y.shape[1]):
         p = np.bincount(y_pred[:, k]) / float(len(X))
-        assert_almost_equal(p[1], 3. / 5, decimal=1)
-        assert_almost_equal(p[0], 1. / 5, decimal=1)
-        assert_almost_equal(p[4], 1. / 5, decimal=1)
+        assert_almost_equal(p[1], 3.0 / 5, decimal=1)
+        assert_almost_equal(p[0], 1.0 / 5, decimal=1)
+        assert_almost_equal(p[4], 1.0 / 5, decimal=1)
 
 
-def test_most_frequent_and_prior_strategy_sparse_target():
+@pytest.mark.parametrize("csc_container", CSC_CONTAINERS)
+def test_most_frequent_and_prior_strategy_sparse_target(csc_container):
     X = [[0]] * 5  # ignored
-    y = sp.csc_matrix(np.array([[1, 0],
-                                [1, 3],
-                                [4, 0],
-                                [0, 1],
-                                [1, 0]]))
+    y = csc_container(np.array([[1, 0], [1, 3], [4, 0], [0, 1], [1, 0]]))
 
     n_samples = len(X)
     y_expected = np.hstack([np.ones((n_samples, 1)), np.zeros((n_samples, 1))])
@@ -654,8 +599,8 @@ def test_most_frequent_and_prior_strategy_sparse_target():
         assert_array_equal(y_pred.toarray(), y_expected)
 
 
-def test_dummy_regressor_sample_weight(n_samples=10):
-    random_state = np.random.RandomState(seed=1)
+def test_dummy_regressor_sample_weight(global_random_seed, n_samples=10):
+    random_state = np.random.RandomState(seed=global_random_seed)
 
     X = [[0]] * n_samples
     y = random_state.rand(n_samples)
@@ -665,15 +610,14 @@ def test_dummy_regressor_sample_weight(n_samples=10):
     assert est.constant_ == np.average(y, weights=sample_weight)
 
     est = DummyRegressor(strategy="median").fit(X, y, sample_weight)
-    assert est.constant_ == _weighted_percentile(y, sample_weight, 50.)
+    assert est.constant_ == _weighted_percentile(y, sample_weight, 50.0)
 
-    est = DummyRegressor(strategy="quantile", quantile=.95).fit(X, y,
-                                                                sample_weight)
-    assert est.constant_ == _weighted_percentile(y, sample_weight, 95.)
+    est = DummyRegressor(strategy="quantile", quantile=0.95).fit(X, y, sample_weight)
+    assert est.constant_ == _weighted_percentile(y, sample_weight, 95.0)
 
 
 def test_dummy_regressor_on_3D_array():
-    X = np.array([[['foo']], [['bar']], [['baz']]])
+    X = np.array([[["foo"]], [["bar"]], [["baz"]]])
     y = np.array([2, 2, 2])
     y_expected = np.array([2, 2, 2])
     cls = DummyRegressor()
@@ -683,7 +627,7 @@ def test_dummy_regressor_on_3D_array():
 
 
 def test_dummy_classifier_on_3D_array():
-    X = np.array([[['foo']], [['bar']], [['baz']]])
+    X = np.array([[["foo"]], [["bar"]], [["baz"]]])
     y = [2, 2, 2]
     y_expected = [2, 2, 2]
     y_proba_expected = [[1], [1], [1]]
@@ -708,28 +652,20 @@ def test_dummy_regressor_return_std():
     assert_array_equal(y_pred_list[1], y_std_expected)
 
 
-@pytest.mark.filterwarnings('ignore: The default value of multioutput')  # 0.23
-@pytest.mark.parametrize("y,y_test", [
-    ([1, 1, 1, 2], [1.25] * 4),
-    (np.array([[2, 2],
-               [1, 1],
-               [1, 1],
-               [1, 1]]),
-     [[1.25, 1.25]] * 4)
-
-])
+@pytest.mark.parametrize(
+    "y,y_test",
+    [
+        ([1, 1, 1, 2], [1.25] * 4),
+        (np.array([[2, 2], [1, 1], [1, 1], [1, 1]]), [[1.25, 1.25]] * 4),
+    ],
+)
 def test_regressor_score_with_None(y, y_test):
     reg = DummyRegressor()
     reg.fit(None, y)
     assert reg.score(None, y_test) == 1.0
 
 
-@pytest.mark.parametrize("strategy", [
-    "mean",
-    "median",
-    "quantile",
-    "constant"
-])
+@pytest.mark.parametrize("strategy", ["mean", "median", "quantile", "constant"])
 def test_regressor_prediction_independent_of_X(strategy):
     y = [0, 2, 1, 1]
     X1 = [[0]] * 4
@@ -755,22 +691,3 @@ def test_dtype_of_classifier_probas(strategy):
     probas = model.fit(X, y).predict_proba(X)
 
     assert probas.dtype == np.float64
-
-
-@pytest.mark.parametrize("Dummy", (DummyRegressor, DummyClassifier))
-def test_outputs_2d_deprecation(Dummy):
-    X = [[1, 2]]
-    y = [0]
-    with pytest.warns(FutureWarning,
-                      match="will be removed in version 0.24"):
-        Dummy().fit(X, y).outputs_2d_
-
-
-# TODO: Remove in 0.24 when DummyClassifier's `strategy` default updates
-def test_strategy_stratified_deprecated_for_prior():
-    X, y = [[1, 2]], [0]
-
-    msg = ("The default value of strategy will change from "
-           "stratified to prior in 0.24")
-    with pytest.warns(FutureWarning, match=msg):
-        DummyClassifier().fit(X, y)
