@@ -1,44 +1,47 @@
+"""
+The :mod:`sklearn.utils.multiclass` module includes utilities to handle
+multiclass/multioutput target in classifiers.
+"""
+
 # Author: Arnaud Joly, Joel Nothman, Hamzeh Alsalhi
 #
 # License: BSD 3 clause
-"""
-Multi-class / multi-label utility function
-==========================================
-
-"""
+import warnings
 from collections.abc import Sequence
 from itertools import chain
 
-from scipy.sparse import issparse
-from scipy.sparse.base import spmatrix
-from scipy.sparse import dok_matrix
-from scipy.sparse import lil_matrix
-
 import numpy as np
+from scipy.sparse import issparse
 
-from .validation import check_array, _assert_all_finite
+from ..utils._array_api import get_namespace
+from ..utils.fixes import VisibleDeprecationWarning
+from .validation import _assert_all_finite, check_array
 
 
 def _unique_multiclass(y):
-    if hasattr(y, '__array__'):
-        return np.unique(np.asarray(y))
+    xp, is_array_api_compliant = get_namespace(y)
+    if hasattr(y, "__array__") or is_array_api_compliant:
+        return xp.unique_values(xp.asarray(y))
     else:
         return set(y)
 
 
 def _unique_indicator(y):
-    return np.arange(check_array(y, ['csr', 'csc', 'coo']).shape[1])
+    xp, _ = get_namespace(y)
+    return xp.arange(
+        check_array(y, input_name="y", accept_sparse=["csr", "csc", "coo"]).shape[1]
+    )
 
 
 _FN_UNIQUE_LABELS = {
-    'binary': _unique_multiclass,
-    'multiclass': _unique_multiclass,
-    'multilabel-indicator': _unique_indicator,
+    "binary": _unique_multiclass,
+    "multiclass": _unique_multiclass,
+    "multilabel-indicator": _unique_indicator,
 }
 
 
 def unique_labels(*ys):
-    """Extract an ordered array of unique labels
+    """Extract an ordered array of unique labels.
 
     We don't allow:
         - mix of multilabel and multiclass (single label) targets
@@ -52,10 +55,11 @@ def unique_labels(*ys):
     Parameters
     ----------
     *ys : array-likes
+        Label values.
 
     Returns
     -------
-    out : numpy array of shape [n_unique_labels]
+    out : ndarray of shape (n_unique_labels,)
         An ordered array of unique labels.
 
     Examples
@@ -68,8 +72,9 @@ def unique_labels(*ys):
     >>> unique_labels([1, 2, 10], [5, 11])
     array([ 1,  2,  5, 10, 11])
     """
+    xp, is_array_api_compliant = get_namespace(*ys)
     if not ys:
-        raise ValueError('No argument has been passed.')
+        raise ValueError("No argument has been passed.")
     # Check that we don't mix label format
 
     ys_types = set(type_of_target(x) for x in ys)
@@ -82,41 +87,55 @@ def unique_labels(*ys):
     label_type = ys_types.pop()
 
     # Check consistency for the indicator format
-    if (label_type == "multilabel-indicator" and
-            len(set(check_array(y, ['csr', 'csc', 'coo']).shape[1]
-                    for y in ys)) > 1):
-        raise ValueError("Multi-label binary indicator input with "
-                         "different numbers of labels")
+    if (
+        label_type == "multilabel-indicator"
+        and len(
+            set(
+                check_array(y, accept_sparse=["csr", "csc", "coo"]).shape[1] for y in ys
+            )
+        )
+        > 1
+    ):
+        raise ValueError(
+            "Multi-label binary indicator input with different numbers of labels"
+        )
 
     # Get the unique set of labels
     _unique_labels = _FN_UNIQUE_LABELS.get(label_type, None)
     if not _unique_labels:
         raise ValueError("Unknown label type: %s" % repr(ys))
 
-    ys_labels = set(chain.from_iterable(_unique_labels(y) for y in ys))
+    if is_array_api_compliant:
+        # array_api does not allow for mixed dtypes
+        unique_ys = xp.concat([_unique_labels(y) for y in ys])
+        return xp.unique_values(unique_ys)
 
+    ys_labels = set(chain.from_iterable((i for i in _unique_labels(y)) for y in ys))
     # Check that we don't mix string type with number type
-    if (len(set(isinstance(label, str) for label in ys_labels)) > 1):
+    if len(set(isinstance(label, str) for label in ys_labels)) > 1:
         raise ValueError("Mix of label input types (string and number)")
 
-    return np.array(sorted(ys_labels))
+    return xp.asarray(sorted(ys_labels))
 
 
 def _is_integral_float(y):
-    return y.dtype.kind == 'f' and np.all(y.astype(int) == y)
+    xp, is_array_api_compliant = get_namespace(y)
+    return xp.isdtype(y.dtype, "real floating") and bool(
+        xp.all(xp.astype((xp.astype(y, xp.int64)), y.dtype) == y)
+    )
 
 
 def is_multilabel(y):
-    """ Check if ``y`` is in a multilabel format.
+    """Check if ``y`` is in a multilabel format.
 
     Parameters
     ----------
-    y : numpy array of shape [n_samples]
+    y : ndarray of shape (n_samples,)
         Target values.
 
     Returns
     -------
-    out : bool,
+    out : bool
         Return ``True``, if ``y`` is in a multilabel format, else ```False``.
 
     Examples
@@ -134,22 +153,49 @@ def is_multilabel(y):
     >>> is_multilabel(np.array([[1, 0, 0]]))
     True
     """
-    if hasattr(y, '__array__') or isinstance(y, Sequence):
-        y = np.asarray(y)
+    xp, is_array_api_compliant = get_namespace(y)
+    if hasattr(y, "__array__") or isinstance(y, Sequence) or is_array_api_compliant:
+        # DeprecationWarning will be replaced by ValueError, see NEP 34
+        # https://numpy.org/neps/nep-0034-infer-dtype-is-object.html
+        check_y_kwargs = dict(
+            accept_sparse=True,
+            allow_nd=True,
+            force_all_finite=False,
+            ensure_2d=False,
+            ensure_min_samples=0,
+            ensure_min_features=0,
+        )
+        with warnings.catch_warnings():
+            warnings.simplefilter("error", VisibleDeprecationWarning)
+            try:
+                y = check_array(y, dtype=None, **check_y_kwargs)
+            except (VisibleDeprecationWarning, ValueError) as e:
+                if str(e).startswith("Complex data not supported"):
+                    raise
+
+                # dtype=object should be provided explicitly for ragged arrays,
+                # see NEP 34
+                y = check_array(y, dtype=object, **check_y_kwargs)
+
     if not (hasattr(y, "shape") and y.ndim == 2 and y.shape[1] > 1):
         return False
 
     if issparse(y):
-        if isinstance(y, (dok_matrix, lil_matrix)):
+        if y.format in ("dok", "lil"):
             y = y.tocsr()
-        return (len(y.data) == 0 or np.unique(y.data).size == 1 and
-                (y.dtype.kind in 'biu' or  # bool, int, uint
-                 _is_integral_float(np.unique(y.data))))
+        labels = xp.unique_values(y.data)
+        return (
+            len(y.data) == 0
+            or (labels.size == 1 or (labels.size == 2) and (0 in labels))
+            and (y.dtype.kind in "biu" or _is_integral_float(labels))  # bool, int, uint
+        )
     else:
-        labels = np.unique(y)
+        labels = xp.unique_values(y)
 
-        return len(labels) < 3 and (y.dtype.kind in 'biu' or  # bool, int, uint
-                                    _is_integral_float(labels))
+        return labels.shape[0] < 3 and (
+            xp.isdtype(y.dtype, ("bool", "signed integer", "unsigned integer"))
+            or _is_integral_float(labels)
+        )
 
 
 def check_classification_targets(y):
@@ -162,14 +208,24 @@ def check_classification_targets(y):
     Parameters
     ----------
     y : array-like
+        Target values.
     """
-    y_type = type_of_target(y)
-    if y_type not in ['binary', 'multiclass', 'multiclass-multioutput',
-                      'multilabel-indicator', 'multilabel-sequences']:
-        raise ValueError("Unknown label type: %r" % y_type)
+    y_type = type_of_target(y, input_name="y")
+    if y_type not in [
+        "binary",
+        "multiclass",
+        "multiclass-multioutput",
+        "multilabel-indicator",
+        "multilabel-sequences",
+    ]:
+        raise ValueError(
+            f"Unknown label type: {y_type}. Maybe you are trying to fit a "
+            "classifier, which expects discrete classes on a "
+            "regression target with continuous values."
+        )
 
 
-def type_of_target(y):
+def type_of_target(y, input_name=""):
     """Determine the type of data indicated by the target.
 
     Note that this type is the most specific type that can be inferred.
@@ -183,11 +239,18 @@ def type_of_target(y):
 
     Parameters
     ----------
-    y : array-like
+    y : {array-like, sparse matrix}
+        Target values. If a sparse matrix, `y` is expected to be a
+        CSR/CSC matrix.
+
+    input_name : str, default=""
+        The data name used to construct the error message.
+
+        .. versionadded:: 1.1.0
 
     Returns
     -------
-    target_type : string
+    target_type : str
         One of:
 
         * 'continuous': `y` is an array-like of floats that are not all
@@ -209,6 +272,7 @@ def type_of_target(y):
 
     Examples
     --------
+    >>> from sklearn.utils.multiclass import type_of_target
     >>> import numpy as np
     >>> type_of_target([0.1, 0.6])
     'continuous'
@@ -233,65 +297,110 @@ def type_of_target(y):
     >>> type_of_target(np.array([[0, 1], [1, 1]]))
     'multilabel-indicator'
     """
-    valid = ((isinstance(y, (Sequence, spmatrix)) or hasattr(y, '__array__'))
-             and not isinstance(y, str))
+    xp, is_array_api_compliant = get_namespace(y)
+    valid = (
+        (isinstance(y, Sequence) or issparse(y) or hasattr(y, "__array__"))
+        and not isinstance(y, str)
+        or is_array_api_compliant
+    )
 
     if not valid:
-        raise ValueError('Expected array-like (array or non-string sequence), '
-                         'got %r' % y)
+        raise ValueError(
+            "Expected array-like (array or non-string sequence), got %r" % y
+        )
 
-    sparse_pandas = (y.__class__.__name__ in ['SparseSeries', 'SparseArray'])
+    sparse_pandas = y.__class__.__name__ in ["SparseSeries", "SparseArray"]
     if sparse_pandas:
         raise ValueError("y cannot be class 'SparseSeries' or 'SparseArray'")
 
     if is_multilabel(y):
-        return 'multilabel-indicator'
+        return "multilabel-indicator"
 
-    try:
-        y = np.asarray(y)
-    except ValueError:
-        # Known to fail in numpy 1.3 for array of arrays
-        return 'unknown'
+    # DeprecationWarning will be replaced by ValueError, see NEP 34
+    # https://numpy.org/neps/nep-0034-infer-dtype-is-object.html
+    # We therefore catch both deprecation (NumPy < 1.24) warning and
+    # value error (NumPy >= 1.24).
+    check_y_kwargs = dict(
+        accept_sparse=True,
+        allow_nd=True,
+        force_all_finite=False,
+        ensure_2d=False,
+        ensure_min_samples=0,
+        ensure_min_features=0,
+    )
+
+    with warnings.catch_warnings():
+        warnings.simplefilter("error", VisibleDeprecationWarning)
+        if not issparse(y):
+            try:
+                y = check_array(y, dtype=None, **check_y_kwargs)
+            except (VisibleDeprecationWarning, ValueError) as e:
+                if str(e).startswith("Complex data not supported"):
+                    raise
+
+                # dtype=object should be provided explicitly for ragged arrays,
+                # see NEP 34
+                y = check_array(y, dtype=object, **check_y_kwargs)
 
     # The old sequence of sequences format
     try:
-        if (not hasattr(y[0], '__array__') and isinstance(y[0], Sequence)
-                and not isinstance(y[0], str)):
-            raise ValueError('You appear to be using a legacy multi-label data'
-                             ' representation. Sequence of sequences are no'
-                             ' longer supported; use a binary array or sparse'
-                             ' matrix instead - the MultiLabelBinarizer'
-                             ' transformer can convert to this format.')
+        first_row = y[[0], :] if issparse(y) else y[0]
+        if (
+            not hasattr(first_row, "__array__")
+            and isinstance(first_row, Sequence)
+            and not isinstance(first_row, str)
+        ):
+            raise ValueError(
+                "You appear to be using a legacy multi-label data"
+                " representation. Sequence of sequences are no"
+                " longer supported; use a binary array or sparse"
+                " matrix instead - the MultiLabelBinarizer"
+                " transformer can convert to this format."
+            )
     except IndexError:
         pass
 
     # Invalid inputs
-    if y.ndim > 2 or (y.dtype == object and len(y) and
-                      not isinstance(y.flat[0], str)):
-        return 'unknown'  # [[[1, 2]]] or [obj_1] and not ["label_1"]
-
-    if y.ndim == 2 and y.shape[1] == 0:
-        return 'unknown'  # [[]]
+    if y.ndim not in (1, 2):
+        # Number of dimension greater than 2: [[[1, 2]]]
+        return "unknown"
+    if not min(y.shape):
+        # Empty ndarray: []/[[]]
+        if y.ndim == 1:
+            # 1-D empty array: []
+            return "binary"  # []
+        # 2-D empty array: [[]]
+        return "unknown"
+    if not issparse(y) and y.dtype == object and not isinstance(y.flat[0], str):
+        # [obj_1] and not ["label_1"]
+        return "unknown"
 
+    # Check if multioutput
     if y.ndim == 2 and y.shape[1] > 1:
         suffix = "-multioutput"  # [[1, 2], [1, 2]]
     else:
         suffix = ""  # [1, 2, 3] or [[1], [2], [3]]
 
-    # check float and contains non-integer float values
-    if y.dtype.kind == 'f' and np.any(y != y.astype(int)):
+    # Check float and contains non-integer float values
+    if xp.isdtype(y.dtype, "real floating"):
         # [.1, .2, 3] or [[.1, .2, 3]] or [[1., .2]] and not [1., 2., 3.]
-        _assert_all_finite(y)
-        return 'continuous' + suffix
+        data = y.data if issparse(y) else y
+        if xp.any(data != xp.astype(data, int)):
+            _assert_all_finite(data, input_name=input_name)
+            return "continuous" + suffix
 
-    if (len(np.unique(y)) > 2) or (y.ndim >= 2 and len(y[0]) > 1):
-        return 'multiclass' + suffix  # [1, 2, 3] or [[1., 2., 3]] or [[1, 2]]
+    # Check multiclass
+    if issparse(first_row):
+        first_row = first_row.data
+    if xp.unique_values(y).shape[0] > 2 or (y.ndim == 2 and len(first_row) > 1):
+        # [1, 2, 3] or [[1., 2., 3]] or [[1, 2]]
+        return "multiclass" + suffix
     else:
-        return 'binary'  # [1, 2] or [["a"], ["b"]]
+        return "binary"  # [1, 2] or [["a"], ["b"]]
 
 
 def _check_partial_fit_first_call(clf, classes=None):
-    """Private helper function for factorizing common classes param logic
+    """Private helper function for factorizing common classes param logic.
 
     Estimators that implement the ``partial_fit`` API need to be provided with
     the list of possible classes at the first call to partial_fit.
@@ -304,16 +413,16 @@ def _check_partial_fit_first_call(clf, classes=None):
     set on ``clf``.
 
     """
-    if getattr(clf, 'classes_', None) is None and classes is None:
-        raise ValueError("classes must be passed on the first call "
-                         "to partial_fit.")
+    if getattr(clf, "classes_", None) is None and classes is None:
+        raise ValueError("classes must be passed on the first call to partial_fit.")
 
     elif classes is not None:
-        if getattr(clf, 'classes_', None) is not None:
+        if getattr(clf, "classes_", None) is not None:
             if not np.array_equal(clf.classes_, unique_labels(classes)):
                 raise ValueError(
                     "`classes=%r` is not the same as on last call "
-                    "to partial_fit, was: %r" % (classes, clf.classes_))
+                    "to partial_fit, was: %r" % (classes, clf.classes_)
+                )
 
         else:
             # This is the first call to partial_fit
@@ -326,11 +435,11 @@ def _check_partial_fit_first_call(clf, classes=None):
 
 
 def class_distribution(y, sample_weight=None):
-    """Compute class priors from multioutput-multiclass target data
+    """Compute class priors from multioutput-multiclass target data.
 
     Parameters
     ----------
-    y : array like or sparse matrix of size (n_samples, n_outputs)
+    y : {array-like, sparse matrix} of size (n_samples, n_outputs)
         The labels for each example.
 
     sample_weight : array-like of shape (n_samples,), default=None
@@ -338,15 +447,14 @@ def class_distribution(y, sample_weight=None):
 
     Returns
     -------
-    classes : list of size n_outputs of arrays of size (n_classes,)
+    classes : list of size n_outputs of ndarray of size (n_classes,)
         List of classes for each column.
 
-    n_classes : list of integers of size n_outputs
-        Number of classes in each column
+    n_classes : list of int of size n_outputs
+        Number of classes in each column.
 
-    class_prior : list of size n_outputs of arrays of size (n_classes,)
+    class_prior : list of size n_outputs of ndarray of size (n_classes,)
         Class distribution of each column.
-
     """
     classes = []
     n_classes = []
@@ -361,18 +469,18 @@ def class_distribution(y, sample_weight=None):
         y_nnz = np.diff(y.indptr)
 
         for k in range(n_outputs):
-            col_nonzero = y.indices[y.indptr[k]:y.indptr[k + 1]]
+            col_nonzero = y.indices[y.indptr[k] : y.indptr[k + 1]]
             # separate sample weights for zero and non-zero elements
             if sample_weight is not None:
                 nz_samp_weight = sample_weight[col_nonzero]
-                zeros_samp_weight_sum = (np.sum(sample_weight) -
-                                         np.sum(nz_samp_weight))
+                zeros_samp_weight_sum = np.sum(sample_weight) - np.sum(nz_samp_weight)
             else:
                 nz_samp_weight = None
                 zeros_samp_weight_sum = y.shape[0] - y_nnz[k]
 
-            classes_k, y_k = np.unique(y.data[y.indptr[k]:y.indptr[k + 1]],
-                                       return_inverse=True)
+            classes_k, y_k = np.unique(
+                y.data[y.indptr[k] : y.indptr[k + 1]], return_inverse=True
+            )
             class_prior_k = np.bincount(y_k, weights=nz_samp_weight)
 
             # An explicit zero was found, combine its weight with the weight
@@ -384,8 +492,7 @@ def class_distribution(y, sample_weight=None):
             # class_prior, make an entry for it
             if 0 not in classes_k and y_nnz[k] < y.shape[0]:
                 classes_k = np.insert(classes_k, 0, 0)
-                class_prior_k = np.insert(class_prior_k, 0,
-                                          zeros_samp_weight_sum)
+                class_prior_k = np.insert(class_prior_k, 0, zeros_samp_weight_sum)
 
             classes.append(classes_k)
             n_classes.append(classes_k.shape[0])
@@ -409,16 +516,16 @@ def _ovr_decision_function(predictions, confidences, n_classes):
 
     Parameters
     ----------
-    predictions : array-like, shape (n_samples, n_classifiers)
+    predictions : array-like of shape (n_samples, n_classifiers)
         Predicted classes for each binary classifier.
 
-    confidences : array-like, shape (n_samples, n_classifiers)
+    confidences : array-like of shape (n_samples, n_classifiers)
         Decision functions or predicted probabilities for positive class
         for each binary classifier.
 
     n_classes : int
         Number of classes. n_classifiers must be
-        ``n_classes * (n_classes - 1 ) / 2``
+        ``n_classes * (n_classes - 1 ) / 2``.
     """
     n_samples = predictions.shape[0]
     votes = np.zeros((n_samples, n_classes))
@@ -440,6 +547,7 @@ def _ovr_decision_function(predictions, confidences, n_classes):
     # The motivation is to use confidence levels as a way to break ties in
     # the votes without switching any decision made based on a difference
     # of 1 vote.
-    transformed_confidences = (sum_of_confidences /
-                               (3 * (np.abs(sum_of_confidences) + 1)))
+    transformed_confidences = sum_of_confidences / (
+        3 * (np.abs(sum_of_confidences) + 1)
+    )
     return votes + transformed_confidences
