@@ -1,38 +1,42 @@
-import warnings
-import unittest
-import sys
-import os
 import atexit
+import os
+import unittest
+import warnings
 
 import numpy as np
-
+import pytest
 from scipy import sparse
 
-import pytest
-
-from sklearn.utils.deprecation import deprecated
-from sklearn.utils.metaestimators import available_if, if_delegate_has_method
-from sklearn.utils._readonly_array_wrapper import _test_sum
-from sklearn.utils._testing import (
-    assert_raises,
-    assert_warns,
-    assert_no_warnings,
-    set_random_state,
-    assert_raise_message,
-    ignore_warnings,
-    check_docstring_parameters,
-    assert_allclose_dense_sparse,
-    assert_raises_regex,
-    TempMemmap,
-    create_memmap_backed_data,
-    _delete_folder,
-    _convert_container,
-    raises,
-    assert_allclose,
-)
-
-from sklearn.tree import DecisionTreeClassifier
 from sklearn.discriminant_analysis import LinearDiscriminantAnalysis
+from sklearn.tree import DecisionTreeClassifier
+from sklearn.utils import _IS_WASM
+from sklearn.utils._testing import (
+    TempMemmap,
+    _convert_container,
+    _delete_folder,
+    _get_warnings_filters_info_list,
+    assert_allclose,
+    assert_allclose_dense_sparse,
+    assert_no_warnings,
+    assert_raise_message,
+    assert_raises,
+    assert_raises_regex,
+    assert_run_python_script_without_output,
+    check_docstring_parameters,
+    create_memmap_backed_data,
+    ignore_warnings,
+    raises,
+    set_random_state,
+    turn_warnings_into_errors,
+)
+from sklearn.utils.deprecation import deprecated
+from sklearn.utils.fixes import (
+    CSC_CONTAINERS,
+    CSR_CONTAINERS,
+    parse_version,
+    sp_version,
+)
+from sklearn.utils.metaestimators import available_if
 
 
 def test_set_random_state():
@@ -44,10 +48,11 @@ def test_set_random_state():
     assert tree.random_state == 3
 
 
-def test_assert_allclose_dense_sparse():
+@pytest.mark.parametrize("csr_container", CSC_CONTAINERS)
+def test_assert_allclose_dense_sparse(csr_container):
     x = np.arange(9).reshape(3, 3)
     msg = "Not equal to tolerance "
-    y = sparse.csc_matrix(x)
+    y = csr_container(x)
     for X in [x, y]:
         # basic compare
         with pytest.raises(AssertionError, match=msg):
@@ -58,7 +63,7 @@ def test_assert_allclose_dense_sparse():
         assert_allclose_dense_sparse(x, y)
 
     A = sparse.diags(np.ones(5), offsets=0).tocsr()
-    B = sparse.csr_matrix(np.ones((1, 5)))
+    B = csr_container(np.ones((1, 5)))
     with pytest.raises(AssertionError, match="Arrays are not equal"):
         assert_allclose_dense_sparse(B, A)
 
@@ -123,10 +128,18 @@ def test_ignore_warning():
     assert_no_warnings(ignore_warnings(_warning_function, category=DeprecationWarning))
     with pytest.warns(DeprecationWarning):
         ignore_warnings(_warning_function, category=UserWarning)()
-    with pytest.warns(UserWarning):
+
+    with pytest.warns() as record:
         ignore_warnings(_multiple_warning_function, category=FutureWarning)()
-    with pytest.warns(DeprecationWarning):
+    assert len(record) == 2
+    assert isinstance(record[0].message, DeprecationWarning)
+    assert isinstance(record[1].message, UserWarning)
+
+    with pytest.warns() as record:
         ignore_warnings(_multiple_warning_function, category=UserWarning)()
+    assert len(record) == 1
+    assert isinstance(record[0].message, DeprecationWarning)
+
     assert_no_warnings(
         ignore_warnings(_warning_function, category=(DeprecationWarning, UserWarning))
     )
@@ -223,45 +236,10 @@ class TestWarns(unittest.TestCase):
             warnings.warn("yo")
             return 3
 
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", UserWarning)
-            filters_orig = warnings.filters[:]
-
-            # TODO: remove in 1.2
-            with pytest.warns(FutureWarning):
-                assert assert_warns(UserWarning, f) == 3
-
-            # test that assert_warns doesn't have side effects on warnings
-            # filters
-            assert warnings.filters == filters_orig
         with pytest.raises(AssertionError):
             assert_no_warnings(f)
         assert assert_no_warnings(lambda x: x, 1) == 1
 
-    # TODO: remove in 1.2
-    @ignore_warnings(category=FutureWarning)
-    def test_warn_wrong_warning(self):
-        def f():
-            warnings.warn("yo", FutureWarning)
-
-        failed = False
-        filters = sys.modules["warnings"].filters[:]
-        try:
-            try:
-                # Should raise an AssertionError
-
-                # assert_warns has a special handling of "FutureWarning" that
-                # pytest.warns does not have
-                assert_warns(UserWarning, f)
-                failed = True
-            except AssertionError:
-                pass
-        finally:
-            sys.modules["warnings"].filters = filters
-
-        if failed:
-            raise AssertionError("wrong warning caught by assert_warn")
-
 
 # Tests for docstrings:
 
@@ -468,64 +446,7 @@ class MockMetaEstimator:
         """Incorrect docstring but should not be tested"""
 
 
-class MockMetaEstimatorDeprecatedDelegation:
-    def __init__(self, delegate):
-        """MetaEstimator to check if doctest on delegated methods work.
-
-        Parameters
-        ---------
-        delegate : estimator
-            Delegated estimator.
-        """
-        self.delegate = delegate
-
-    @if_delegate_has_method(delegate="delegate")
-    def predict(self, X):
-        """This is available only if delegate has predict.
-
-        Parameters
-        ----------
-        y : ndarray
-            Parameter y
-        """
-        return self.delegate.predict(X)
-
-    @if_delegate_has_method(delegate="delegate")
-    @deprecated("Testing a deprecated delegated method")
-    def score(self, X):
-        """This is available only if delegate has score.
-
-        Parameters
-        ---------
-        y : ndarray
-            Parameter y
-        """
-
-    @if_delegate_has_method(delegate="delegate")
-    def predict_proba(self, X):
-        """This is available only if delegate has predict_proba.
-
-        Parameters
-        ---------
-        X : ndarray
-            Parameter X
-        """
-        return X
-
-    @deprecated("Testing deprecated function with wrong params")
-    def fit(self, X, y):
-        """Incorrect docstring but should not be tested"""
-
-
-@pytest.mark.filterwarnings("ignore:if_delegate_has_method was deprecated")
-@pytest.mark.parametrize(
-    "mock_meta",
-    [
-        MockMetaEstimator(delegate=MockEst()),
-        MockMetaEstimatorDeprecatedDelegation(delegate=MockEst()),
-    ],
-)
-def test_check_docstring_parameters(mock_meta):
+def test_check_docstring_parameters():
     pytest.importorskip(
         "numpydoc",
         reason="numpydoc is required to test the docstrings",
@@ -544,21 +465,30 @@ def test_check_docstring_parameters(mock_meta):
         check_docstring_parameters(Klass.f_bad_sections)
 
     incorrect = check_docstring_parameters(f_check_param_definition)
+    mock_meta = MockMetaEstimator(delegate=MockEst())
     mock_meta_name = mock_meta.__class__.__name__
     assert incorrect == [
-        "sklearn.utils.tests.test_testing.f_check_param_definition There "
-        "was no space between the param name and colon ('a: int')",
-        "sklearn.utils.tests.test_testing.f_check_param_definition There "
-        "was no space between the param name and colon ('b:')",
-        "sklearn.utils.tests.test_testing.f_check_param_definition There "
-        "was no space between the param name and colon ('d:int')",
+        (
+            "sklearn.utils.tests.test_testing.f_check_param_definition There "
+            "was no space between the param name and colon ('a: int')"
+        ),
+        (
+            "sklearn.utils.tests.test_testing.f_check_param_definition There "
+            "was no space between the param name and colon ('b:')"
+        ),
+        (
+            "sklearn.utils.tests.test_testing.f_check_param_definition There "
+            "was no space between the param name and colon ('d:int')"
+        ),
     ]
 
     messages = [
         [
             "In function: sklearn.utils.tests.test_testing.f_bad_order",
-            "There's a parameter name mismatch in function docstring w.r.t."
-            " function signature, at index 0 diff: 'b' != 'a'",
+            (
+                "There's a parameter name mismatch in function docstring w.r.t."
+                " function signature, at index 0 diff: 'b' != 'a'"
+            ),
             "Full diff:",
             "- ['b', 'a']",
             "+ ['a', 'b']",
@@ -566,8 +496,10 @@ def test_check_docstring_parameters(mock_meta):
         [
             "In function: "
             + "sklearn.utils.tests.test_testing.f_too_many_param_docstring",
-            "Parameters in function docstring have more items w.r.t. function"
-            " signature, first extra item: c",
+            (
+                "Parameters in function docstring have more items w.r.t. function"
+                " signature, first extra item: c"
+            ),
             "Full diff:",
             "- ['a', 'b']",
             "+ ['a', 'b', 'c']",
@@ -575,16 +507,20 @@ def test_check_docstring_parameters(mock_meta):
         ],
         [
             "In function: sklearn.utils.tests.test_testing.f_missing",
-            "Parameters in function docstring have less items w.r.t. function"
-            " signature, first missing item: b",
+            (
+                "Parameters in function docstring have less items w.r.t. function"
+                " signature, first missing item: b"
+            ),
             "Full diff:",
             "- ['a', 'b']",
             "+ ['a']",
         ],
         [
             "In function: sklearn.utils.tests.test_testing.Klass.f_missing",
-            "Parameters in function docstring have less items w.r.t. function"
-            " signature, first missing item: X",
+            (
+                "Parameters in function docstring have less items w.r.t. function"
+                " signature, first missing item: X"
+            ),
             "Full diff:",
             "- ['X', 'y']",
             "+ []",
@@ -592,8 +528,10 @@ def test_check_docstring_parameters(mock_meta):
         [
             "In function: "
             + f"sklearn.utils.tests.test_testing.{mock_meta_name}.predict",
-            "There's a parameter name mismatch in function docstring w.r.t."
-            " function signature, at index 0 diff: 'X' != 'y'",
+            (
+                "There's a parameter name mismatch in function docstring w.r.t."
+                " function signature, at index 0 diff: 'X' != 'y'"
+            ),
             "Full diff:",
             "- ['X']",
             "?   ^",
@@ -617,8 +555,10 @@ def test_check_docstring_parameters(mock_meta):
         ],
         [
             "In function: " + f"sklearn.utils.tests.test_testing.{mock_meta_name}.fit",
-            "Parameters in function docstring have less items w.r.t. function"
-            " signature, first missing item: X",
+            (
+                "Parameters in function docstring have less items w.r.t. function"
+                " signature, first missing item: X"
+            ),
             "Full diff:",
             "- ['X', 'y']",
             "+ []",
@@ -679,62 +619,35 @@ def test_tempmemmap(monkeypatch):
     assert registration_counter.nb_calls == 2
 
 
-@pytest.mark.parametrize("aligned", [False, True])
-def test_create_memmap_backed_data(monkeypatch, aligned):
+@pytest.mark.xfail(_IS_WASM, reason="memmap not fully supported")
+def test_create_memmap_backed_data(monkeypatch):
     registration_counter = RegistrationCounter()
     monkeypatch.setattr(atexit, "register", registration_counter)
 
     input_array = np.ones(3)
-    data = create_memmap_backed_data(input_array, aligned=aligned)
+    data = create_memmap_backed_data(input_array)
     check_memmap(input_array, data)
     assert registration_counter.nb_calls == 1
 
-    data, folder = create_memmap_backed_data(
-        input_array, return_folder=True, aligned=aligned
-    )
+    data, folder = create_memmap_backed_data(input_array, return_folder=True)
     check_memmap(input_array, data)
     assert folder == os.path.dirname(data.filename)
     assert registration_counter.nb_calls == 2
 
     mmap_mode = "r+"
-    data = create_memmap_backed_data(input_array, mmap_mode=mmap_mode, aligned=aligned)
+    data = create_memmap_backed_data(input_array, mmap_mode=mmap_mode)
     check_memmap(input_array, data, mmap_mode)
     assert registration_counter.nb_calls == 3
 
     input_list = [input_array, input_array + 1, input_array + 2]
-    mmap_data_list = create_memmap_backed_data(input_list, aligned=aligned)
+    mmap_data_list = create_memmap_backed_data(input_list)
     for input_array, data in zip(input_list, mmap_data_list):
         check_memmap(input_array, data)
     assert registration_counter.nb_calls == 4
 
-    with pytest.raises(
-        ValueError,
-        match=(
-            "When creating aligned memmap-backed arrays, input must be a single array"
-            " or a sequence of arrays"
-        ),
-    ):
-        create_memmap_backed_data([input_array, "not-an-array"], aligned=True)
-
-
-@pytest.mark.parametrize("dtype", [np.float32, np.float64, np.int32, np.int64])
-def test_memmap_on_contiguous_data(dtype):
-    """Test memory mapped array on contiguous memoryview."""
-    x = np.arange(10).astype(dtype)
-    assert x.flags["C_CONTIGUOUS"]
-    assert x.flags["ALIGNED"]
-
-    # _test_sum consumes contiguous arrays
-    # def _test_sum(NUM_TYPES[::1] x):
-    sum_origin = _test_sum(x)
-
-    # now on memory mapped data
-    # aligned=True so avoid https://github.com/joblib/joblib/issues/563
-    # without alignment, this can produce segmentation faults, see
-    # https://github.com/scikit-learn/scikit-learn/pull/21654
-    x_mmap = create_memmap_backed_data(x, mmap_mode="r+", aligned=True)
-    sum_mmap = _test_sum(x_mmap)
-    assert sum_mmap == pytest.approx(sum_origin, rel=1e-11)
+    output_data, other = create_memmap_backed_data([input_array, "not-an-array"])
+    check_memmap(input_array, output_data)
+    assert other == "not-an-array"
 
 
 @pytest.mark.parametrize(
@@ -744,8 +657,10 @@ def test_memmap_on_contiguous_data(dtype):
         ("tuple", tuple),
         ("array", np.ndarray),
         ("sparse", sparse.csr_matrix),
-        ("sparse_csr", sparse.csr_matrix),
-        ("sparse_csc", sparse.csc_matrix),
+        # using `zip` will only keep the available sparse containers
+        # depending of the installed SciPy version
+        *zip(["sparse_csr", "sparse_csr_array"], CSR_CONTAINERS),
+        *zip(["sparse_csc", "sparse_csc_array"], CSC_CONTAINERS),
         ("dataframe", lambda: pytest.importorskip("pandas").DataFrame),
         ("series", lambda: pytest.importorskip("pandas").Series),
         ("index", lambda: pytest.importorskip("pandas").Index),
@@ -774,6 +689,7 @@ def test_convert_container(
         # instead of the whole file
         container_type = container_type()
     container = [0, 1]
+
     container_converted = _convert_container(
         container,
         constructor_name,
@@ -791,6 +707,44 @@ def test_convert_container(
         assert container_converted.dtypes[0] == dtype
 
 
+def test_convert_container_categories_pandas():
+    pytest.importorskip("pandas")
+    df = _convert_container(
+        [["x"]], "dataframe", ["A"], categorical_feature_names=["A"]
+    )
+    assert df.dtypes.iloc[0] == "category"
+
+
+def test_convert_container_categories_polars():
+    pl = pytest.importorskip("polars")
+    df = _convert_container([["x"]], "polars", ["A"], categorical_feature_names=["A"])
+    assert df.schema["A"] == pl.Categorical()
+
+
+def test_convert_container_categories_pyarrow():
+    pa = pytest.importorskip("pyarrow")
+    df = _convert_container([["x"]], "pyarrow", ["A"], categorical_feature_names=["A"])
+    assert type(df.schema[0].type) is pa.DictionaryType
+
+
+@pytest.mark.skipif(
+    sp_version >= parse_version("1.8"),
+    reason="sparse arrays are available as of scipy 1.8.0",
+)
+@pytest.mark.parametrize("constructor_name", ["sparse_csr_array", "sparse_csc_array"])
+@pytest.mark.parametrize("dtype", [np.int32, np.int64, np.float32, np.float64])
+def test_convert_container_raise_when_sparray_not_available(constructor_name, dtype):
+    """Check that if we convert to sparse array but sparse array are not supported
+    (scipy<1.8.0), we should raise an explicit error."""
+    container = [0, 1]
+
+    with pytest.raises(
+        ValueError,
+        match=f"only available with scipy>=1.8.0, got {sp_version}",
+    ):
+        _convert_container(container, constructor_name, dtype=dtype)
+
+
 def test_raises():
     # Tests for the raises context manager
 
@@ -877,3 +831,93 @@ def test_float32_aware_assert_allclose():
     with pytest.raises(AssertionError):
         assert_allclose(np.array([1e-5], dtype=np.float32), 0.0)
     assert_allclose(np.array([1e-5], dtype=np.float32), 0.0, atol=2e-5)
+
+
+@pytest.mark.xfail(_IS_WASM, reason="cannot start subprocess")
+def test_assert_run_python_script_without_output():
+    code = "x = 1"
+    assert_run_python_script_without_output(code)
+
+    code = "print('something to stdout')"
+    with pytest.raises(AssertionError, match="Expected no output"):
+        assert_run_python_script_without_output(code)
+
+    code = "print('something to stdout')"
+    with pytest.raises(
+        AssertionError,
+        match="output was not supposed to match.+got.+something to stdout",
+    ):
+        assert_run_python_script_without_output(code, pattern="to.+stdout")
+
+    code = "\n".join(["import sys", "print('something to stderr', file=sys.stderr)"])
+    with pytest.raises(
+        AssertionError,
+        match="output was not supposed to match.+got.+something to stderr",
+    ):
+        assert_run_python_script_without_output(code, pattern="to.+stderr")
+
+
+@pytest.mark.parametrize(
+    "constructor_name",
+    [
+        "sparse_csr",
+        "sparse_csc",
+        pytest.param(
+            "sparse_csr_array",
+            marks=pytest.mark.skipif(
+                sp_version < parse_version("1.8"),
+                reason="sparse arrays are available as of scipy 1.8.0",
+            ),
+        ),
+        pytest.param(
+            "sparse_csc_array",
+            marks=pytest.mark.skipif(
+                sp_version < parse_version("1.8"),
+                reason="sparse arrays are available as of scipy 1.8.0",
+            ),
+        ),
+    ],
+)
+def test_convert_container_sparse_to_sparse(constructor_name):
+    """Non-regression test to check that we can still convert a sparse container
+    from a given format to another format.
+    """
+    X_sparse = sparse.random(10, 10, density=0.1, format="csr")
+    _convert_container(X_sparse, constructor_name)
+
+
+def check_warnings_as_errors(warning_info, warnings_as_errors):
+    if warning_info.action == "error" and warnings_as_errors:
+        with pytest.raises(warning_info.category, match=warning_info.message):
+            warnings.warn(
+                message=warning_info.message,
+                category=warning_info.category,
+            )
+    if warning_info.action == "ignore":
+        with warnings.catch_warnings(record=True) as record:
+            message = warning_info.message
+            # Special treatment when regex is used
+            if "Pyarrow" in message:
+                message = "\nPyarrow will become a required dependency"
+
+            warnings.warn(
+                message=message,
+                category=warning_info.category,
+            )
+            assert len(record) == 0 if warnings_as_errors else 1
+            if record:
+                assert str(record[0].message) == message
+                assert record[0].category == warning_info.category
+
+
+@pytest.mark.parametrize("warning_info", _get_warnings_filters_info_list())
+def test_sklearn_warnings_as_errors(warning_info):
+    warnings_as_errors = os.environ.get("SKLEARN_WARNINGS_AS_ERRORS", "0") != "0"
+    check_warnings_as_errors(warning_info, warnings_as_errors=warnings_as_errors)
+
+
+@pytest.mark.parametrize("warning_info", _get_warnings_filters_info_list())
+def test_turn_warnings_into_errors(warning_info):
+    with warnings.catch_warnings():
+        turn_warnings_into_errors()
+        check_warnings_as_errors(warning_info, warnings_as_errors=True)
